import processing.core.*; 
import processing.data.*; 
import processing.event.*; 
import processing.opengl.*; 

import ddf.minim.*; 

import java.util.HashMap; 
import java.util.ArrayList; 
import java.io.File; 
import java.io.BufferedReader; 
import java.io.PrintWriter; 
import java.io.InputStream; 
import java.io.OutputStream; 
import java.io.IOException; 

public class ping_v7 extends PApplet {

//PING is a prototype made for Intro to Game Design: LiveLessions
//It is open source under a Creative Commons 4.0 Attribution-ShareAlike License (https://creativecommons.org/licenses/by-sa/4.0/legalcode)

//import sound

Minim minim;
AudioSample kick;

//Here's where we initiate all of the main objects in the game; note that they have their own code files
Ground ground;
Player player1;
Player player2;
Ball ball;

boolean start = false;
int padding = 25;
PFont scoreFont;

//setup is a function that we contain all of the initial states of the game in
public void setup() {
   //sound 
  minim = new Minim(this);
  kick = minim.loadSample( "18791.mp3", // filename
                  512      // buffer size
                         );
  
  //-----
  size(640, 480);
  ground = new Ground();
  player1 = new Player(1, 20, height/2, 1);
  player2 = new Player(2, width - 20, height/2, 1);
  player1.stop();
  player2.stop();
  ball = new Ball(0, 0);
  ball.setPos(width/2, height/2);
  scoreFont = createFont("Arial", 40);
  textFont(scoreFont);
}

//draw is another processing function where anything happening on the screen is drawn and updated
public void draw() {
  background(0);
  stroke(255); 
  fill(255);
  ground.draw();
  player1.draw();
  player2.draw();
  ball.draw();
  if (start) {
    
  }else{
   fill(255);
   textAlign(CENTER);
   textSize(64);
   text("PING",width/2,height/2-32);
   textSize(14);
   text("Player 1: 'W' move up, 'S' move down, 'Q' change paddle",width/2,height/2+32);
   text("Player 2:  Narrow Key 'UP' move up, narrow Key'DOWN' move down, '/' change paddle",width/2,height/2+64);
   
   textSize(22);
   text("Press 'SPACE' to Start",width/2,height/2+100);
  }
   

}

public void keyPressed() {
  if (key == 'w' || key == 'W') {
    player1.up();
  }
  if (key == 's' || key == 'S') {
    player1.down();
  }
  
  if (keyCode == UP) {
    player2.up();
  }
  if (keyCode == DOWN) {
    player2.down();
  }
  
  if (key == ' '){
    if(start == false){
        ball.reset();
        player1.reset();
        player2.reset();
        start = true;
    }
  }
}

public void keyReleased() {
  if (key == 'w' || key == 'W' || key == 's' || key == 'S') {
    player1.stop();
  } else if (keyCode == UP || keyCode == DOWN) {
    player2.stop();
  }
}

public void resetGame() {
  player1.setPos(20, height/2);
  player2.setPos(width - 20, height/2);
  player1.resetScore();
  player2.resetScore();
    ball.posX =  width/2;
    ball.posY = height/2;
    ball.movY = 0;
}

class Ball {

  float posX, posY,orgX,orgY, movX, movY, speedX, speedY, size;
 
  float ballLeft;
  float ballRight;
  float ballTop;
  float ballBot;

  Ball(float _posX, float _posY) {
    posX = _posX;
    posY = _posY;
    movX = 3;
    movY = 0;
    speedX = 0;
    speedY = 0;
    size = 10;
   }

  public void setPos(float _posX, float _posY) {
    posX = _posX;
    posY = _posY;
    orgX = posX;
    orgY = posY;
  }

  public void update() {
    posX += movX;
    posY += movY;
   
    //calculate ball edges
    ballLeft = posX-size/2;
    ballRight = posX+size/2;
    ballTop = posY - size/2;
    ballBot = posY + size/2;
    
    //detect boundary 
    checkBoundry();
    
    //left paddel interaction
    if(ballLeft< player1.right && ballRight > player1.left){
      if(ballBot > player1.top && ballTop < player1.bot){
        movX *=-1;
        posX = player1.right + size/2;
        float hit = posY - player1.posY;
        movY += (hit / (player1.size/2)) *2;    
      //reduce the paddle 1 size
        player1.size -=2;
         if(player1.size<10){
          player1.size = 10;
        }
        //play sound
        kick.trigger();
      }
    }
    
     //right paddel interaction
    if(ballRight > player2.left && ballLeft < player2.right){
      if(ballBot > player2.top && ballTop < player2.bot){
        movX *=-1;
        posX = player2.left - size/2;
        float hit = posY - player2.posY;
        movY += (hit / (player2.size/2)) *2; // range -1 to 1 (*2)
        //reduce the paddle 2 size
        player2.size -=2;
        if(player2.size<10){
          player2.size = 10;
        }
        //play sound
        kick.trigger();
      }
    }
    
   }
  
  // reset ball's position
  public void reset(){
     posX = orgX;
     posY = orgY;
     movX = 0;
     movY = 0;
     
     //we use random to change ball's initial direction
     //ball's initial speed
     int speed = 5;
     if(random(100)>50){
        movX = speed;
     }else{
        movX = -speed;
     }
     
  }

  public float getPosY() {
    return posY;
  }
  
  public void draw() {
    update();
    rectMode(CENTER);
    noStroke();
    fill(255);
    rect(posX, posY, size, size);
  }
  
  public void checkBoundry(){
    
    if ( ballTop < padding ){ // check top side
       posY = padding + size/2;
       movY *= -1;
       kick.trigger();
    }
  
    if(ballBot > height-padding ) {  // check bottom side
      movY *= -1;
      posY = height - padding - size/2;
      kick.trigger();
    }
    
    if (posX < 0) { // out left side
       player2.scorePlus();
        player2.size*=2; 
       if(player2.size>80){
          player2.size = 80;
        }  
       reset();
    }
    
     if (posX > width + size/2) { // out right side
       player1.scorePlus(); 
       player1.size*=2; 
       if(player1.size>80){
          player1.size = 80;
        }      
       reset();
    }
  }
}




class Ground {
  
  int scoreP1, scoreP2;
  
  Ground() {
    scoreP1 = 0;
    scoreP2 = 0;
  }
  
  public void draw() {
  // playfield
  stroke(255,0, 200);
  line(padding, padding, width-padding, padding);
  line(padding, height-padding, width-padding, height-padding);
  line(width/2, padding, width/2, height-padding);

  // draw Score
  fill(255,0, 200);
  textAlign(LEFT);
  textSize(16);
  text(player1.getScore(), padding, padding-8); 
  textAlign(RIGHT);
  text(player2.getScore(), width-padding, padding-8);
  }
}
class Player {
  float posX, posY, movX, movY, speedY;
  int id, size, thickness, score, type;
  int difficulty, pause, pauseTime, movPause, dir;
  float currSpeed;
  float top;
  float bot;
  float left;
  float right;
  int rectColor;

  Player(int _id, float _posX, float _posY, int _type) {
    id = _id;
    posX = _posX;
    posY = _posY;
    type = _type;
    size = 80;
    thickness = 10;
    score = 0;
    pause = 75;
    pauseTime = pause;
    movPause = 0;
    dir = 0;
    currSpeed = 3;
    rectColor = color(255);
  }

  public void draw() {
    update();
    noStroke();
   
    fill(rectColor);
    rect(posX, posY, thickness, size);

    posY += speedY;
    //checks to see if the paddle is out of bounds
    if (posY < padding+size/2) {
        posY = padding+size/2;
    } else if (posY > height-padding-size/2) {
       posY = height-padding-size/2;
    }
    
  }

  public void update() {
   // get paddle edges
   top    = posY - size/2;
   bot    = posY + size/2;
   left   = posX - thickness/2;
   right  = posX + thickness/2;
   currSpeed = map(size,0, 80, 10, 3);
  }
  
  public void up() {
    speedY = currSpeed;
    speedY = speedY * -1;
    movY = movY * -1;
  }

  public void down() {
    speedY = currSpeed;
    speedY = speedY * 1;
    movY = movY * 1;
  }

  public void stop() {
    speedY = 0;
  }


  public float getMovY() {
    return movY;
  }

  public void scorePlus() {
    if(start){ 
      score ++;
    }
  }

  public int getScore() {
    return score;
  }

  public void resetScore() {
    score = 0;
  }

  public void setPos(int _posX, int _posY) {
    posX = _posX;
    posY = _posY;
  }
  
  public void reset(){
    size = 80;
  }
}

  static public void main(String[] passedArgs) {
    String[] appletArgs = new String[] { "ping_v7" };
    if (passedArgs != null) {
      PApplet.main(concat(appletArgs, passedArgs));
    } else {
      PApplet.main(appletArgs);
    }
  }
}
